#
# Usage:
#
#   class FancyModel < ActiveRecord::Base
#     include Enscoped
#     named_scope :informative
#     named_scope :entertaining
#     named_scope :brief
#     named_scope :verbose
#     named_scope :boring
#     named_scope :exciting
#     named_scope :free
#     named_scope :expensive
#   end
#   
#   # execute a query
#   FancyModel.enscoped(:informative, :entertaining).count
#   
#   # just the hash condition
#   FancyModel.scope_for_scopes(:free, :expensive)
#
# 
module Enscoped
  
  def self.included(base)
    base.extend(ClassMethods)
  end
  
  module ClassMethods
    
    # Create one named scope from many named scopes.
    #
    # @param [Array<String or Symbol>] scope names
    # @return [Hash] hash compatible with ActiveRecord::Base#scoped
    #
    # Example:
    #
    #  Book.scope_for_scopes(:short, :informative, :free)
    #
    #  %w(short informative free).inject(Hash.new) { |memo, name|
    #    self.scoped(memo).send(name).current_scoped_methods[:find]
    #  }
    #
    def scope_for_scopes(*scopes)
      # intersect valid scope names so dangerous methods
      # like destroy_all aren't invoked!
      safe_scopes = self.scopes.keys & scopes.map(&:to_sym)
    
      # build (and return) a hash of the constructed scope
      safe_scopes.inject({}) { |memo, scope|
        self.scoped(memo).send(scope).current_scoped_methods[:find]
      }
    end
    
    # Creates and executes a named scope for the given scopes.  Ideally, this
    # should do what #scoped does.  I'd like scoped to be called subsequently
    # without executing first.
    #
    # @return [] erm... what it returns and what it *should* return are
    #   different
    def enscoped(*scopes)
      self.scoped(self.scope_for_scopes(*scopes))
    end
    
  end

end
